@@ -503,7 +503,7 @@ class Config:
     def getWriteDirectories(self, app):
         rootDir = self.optionMap.setPathFromOptionsOrEnv("TEXTTEST_TMP", app.getConfigValue("default_texttest_tmp"))  # Location of temporary files from test runs
         if not os.path.isdir(rootDir) and not self.hasWritePermission(os.path.dirname(rootDir)):
-            rootDir = self.optionMap.setPathFromOptionsOrEnv("", "$TEXTTEST_PERSONAL_CONFIG/tmp")
+            rootDir = self.optionMap.normalisePath(os.path.expandvars("$TEXTTEST_PERSONAL_CONFIG/tmp"))
         writeDir = os.path.join(rootDir, self.getWriteDirectoryName(app))
         localRootDir = self.optionMap.getPathFromOptionsOrEnv("TEXTTEST_LOCAL_TMP", app.getConfigValue("default_texttest_local_tmp")) # Location of temporary files on local disk from test runs. Defaults to value of TEXTTEST_TMP
         if localRootDir:
@@ -1568,7 +1568,7 @@ class Config:
                     args[-1] = args[-1].replace(app.writeDirectory, remoteTmp)
                 for i in range(len(args)):
                     # Remote shells cause spaces etc to be interpreted several times
-                    args[i] = args[i].replace(" ", "\ ")
+                    args[i] = args[i].replace(" ", r"\ ")
             return args
 
     def getFullDisplay(self):
