@@ -9,17 +9,18 @@ import time
 import re
 import stat
 import shlex
-import types
 import fnmatch
 import subprocess
-from collections import OrderedDict
+from collections import OrderedDict, deque
 from traceback import format_exception
 from threading import currentThread, RLock
 from queue import Queue, Empty
 from glob import glob
 from datetime import datetime
-from pickle import Unpickler, UnpicklingError
+from pickle import Unpickler
 from locale import getpreferredencoding
+import importlib.resources
+import sre_parse
 
 
 class Callable:
@@ -42,19 +43,11 @@ class Callable:
 
 
 def findInstallationRoots():
-    packageDir = os.path.dirname(__file__)
-    if getattr(sys, 'frozen', False):
-        roots = glob(os.path.join(os.path.dirname(sys.executable), "lib", "python*", "site-packages", "texttestlib"))
-    else:
-        roots = [packageDir]
-    installationRoot = os.path.dirname(packageDir)
-    if os.path.basename(installationRoot) == "generic":
-        siteRoot = os.path.dirname(installationRoot)
-        roots.append(siteRoot)
-    else:
-        siteDir = os.path.join(installationRoot, "site")
-        if os.path.isdir(siteDir):
-            roots.append(siteDir)
+    installationRoot = str(importlib.resources.files("texttestlib"))
+    roots = [ installationRoot ]
+    siteDir = os.path.join(installationRoot, "site")
+    if os.path.isdir(siteDir):
+        roots.append(siteDir)
     return roots
 
 
@@ -232,7 +225,7 @@ def printWarning(message, stdout=False):
 
 
 # Useful stuff to handle regular expressions
-regexChars = re.compile("[\^\$\[\]\{\}\\\*\?\|\+]")
+regexChars = re.compile(r"[\^\$\[\]\{\}\\\*\?\|\+]")
 
 
 def isRegularExpression(text):
@@ -1476,11 +1469,27 @@ class MatchAggregator:
 
     def expand(self, template):
         if self.match is not None:
-            # ?? This is not documented, but has remained unchanged across many Python versions
-            # Better ideas welcome. The match objects are defined in C, the class name isn't public and hence they cannot be subclassed or monkey patched
-            return re._expand(self.patternAggregator, self, template)
+            # Was originally calling re._expand, removed in Python 3.12
+            # Tried to use new parsing and copied expand function below to preserve behaviour
+            parsed_template = sre_parse.parse_template(template, self.patternAggregator)
+            if hasattr(sre_parse, "expand_template"):
+                return sre_parse.expand_template(parsed_template, self)
+            else:
+                return self.expand_template(parsed_template)
         else:
             return template
+        
+    def expand_template(self, template):
+        g = self.group
+        empty = self.string[:0]
+        groups = [ (ix, item) for (ix, item) in enumerate(template) if isinstance(item, int) ]            
+        literals = template[:]
+        try:
+            for index, group in groups:
+                literals[index] = g(group) or empty
+        except IndexError:
+            raise re.error("invalid group reference %d" % index)
+        return empty.join(literals)
 
     def __getattr__(self, name):
         return getattr(self.match, name)
@@ -1560,6 +1569,7 @@ class MultiEntryDictionary(OrderedDict):
         self.importFileFinder = importFileFinder
         self.allowSectionHeaders = allowSectionHeaders
         self.fileTrackSections = fileTrackSections
+        self._currentFiles = deque()
 
     def __reduce__(self):
         # Need this because of __reduce__ in OrderedDict
@@ -1590,6 +1600,7 @@ class MultiEntryDictionary(OrderedDict):
 
     def readFromFile(self, filename, *args, **kwargs):
         self.diag.info("Reading file " + filename)
+        self._currentFiles.append(filename)
         currSectionName = ""
         for line in readList(filename):
             if self.allowSectionHeaders and self.isSectionHeader(line):
@@ -1597,15 +1608,18 @@ class MultiEntryDictionary(OrderedDict):
             elif ":" in line:
                 self.parseConfigLine(line, currSectionName, filename, *args, **kwargs)
             else:
-                self.warn("Could not parse config line " + line)
+                self.warn(f"Could not parse config line '{line}'")
+        self._currentFiles.pop()
 
     def isSectionHeader(self, line):
         return line.startswith("[") and line.endswith("]")
 
     def warn(self, message):
-        if message not in self.warnings:
-            self.warnings.append(message)
-            printWarning(message)
+        currentFile = self._currentFiles[-1] if self._currentFiles else None
+        fullMessage = f"{message} (see {currentFile})" if currentFile else message
+        if fullMessage not in self.warnings:
+            self.warnings.append(fullMessage)
+            printWarning(fullMessage)
 
     def getFileDefining(self, sectionName, entryName, value, envMapping=os.environ):
         valueDict = self.fileTrackSections.get(sectionName)
@@ -1644,12 +1658,12 @@ class MultiEntryDictionary(OrderedDict):
                     self[name] = OrderedDict(value)
                     return name
                 else:
-                    self.warn("Config entry name '" + name + "' incorrectly used as a section marker.")
+                    self.warn(f"Config entry name '{name}' incorrectly used as a section marker.")
             elif insert:
                 self[name] = OrderedDict()
                 return name
             elif errorOnUnknown:
-                self.warn("Config section name '" + name + "' not recognised.")
+                self.warn(f"Config section name '{name}' not recognised.")
         return ""
 
     def addEntry(self, entryName, entry, sectionName="", *args, **kwargs):
@@ -1682,7 +1696,7 @@ class MultiEntryDictionary(OrderedDict):
                 self.diag.info("Inserting " + entryName + "=" + repr(entry))
                 currDict[entryName] = self.castEntry(entryName, entry, currDict)
             elif errorOnUnknown:
-                self.warn("Config entry name '" + entryName + "' not recognised.")
+                self.warn(f"Config entry name '{entryName}' not recognised.")
 
     def getDictionaryValueType(self, currDict):
         val = list(currDict.values())
@@ -1727,7 +1741,7 @@ class MultiEntryDictionary(OrderedDict):
             if "default" in newCurrDict:
                 self.insertEntry("default", entry, newCurrDict)
             else:
-                self.warn("Config entry name '" + entryName + "' is only valid as a section marker.")
+                self.warn(f"Config entry name '{entryName}' is only valid as a section marker.")
         else:
             currDict[entryName] = type(currEntry)(entry)
 
